import os

_DEFAULT_PACT_BROKER_URL = 'http://localhost:8080/'
_DEFAULT_PACT_DIR = '.'
_DEFAULT_PACT_BROKER_USER = None
_DEFAULT_PACT_BROKER_PASSWORD = None

_DEFAULT_AUTHENTICATION = False

PACT_BROKER_URL = os.environ.get(
    'PACT_BROKER_URL', _DEFAULT_PACT_BROKER_URL
).rstrip('/')

PACT_BROKER_USER = os.environ.get(
    'PACT_BROKER_USER', _DEFAULT_PACT_BROKER_USER
)
PACT_BROKER_PASSWORD = os.environ.get(
    'PACT_BROKER_PASSWORD', _DEFAULT_PACT_BROKER_PASSWORD
)

PACT_DIR = os.environ.get(
    'PACT_DIR', _DEFAULT_PACT_DIR
)



import click

from . import BrokerClient
from . import settings


@click.command()
@click.option('--broker_url', help='Pact Broker host.')
@click.option('--consumer', help='Consumer service name.')
@click.option('--provider', help='Provider service name.')
@click.option('--user', help='Pact Broker user.')
@click.option('--password', help='Pact Broker password.')
@click.option('--pact_dir', default='.', help='Directory to save Pacts to.')
@click.option('--pact_version', default='latest', help='Pact version to pull.')
@click.option(
    '--auth',
    default=False,
    is_flag=True,
    help='Indicates if Pact Broker is authenticated.'
)
def pull_pact(
    broker_url,
    consumer,
    provider,
    user,
    password,
    auth,
    pact_dir,
    pact_version
):

    broker_client = BrokerClient(
        broker_url=broker_url or settings.PACT_BROKER_URL,
        pact_dir=pact_dir or settings.PACT_DIR,
        user=user or settings.PACT_BROKER_USER,
        password=password or settings.PACT_BROKER_PASSWORD,
        authentication=auth
    )

    result = broker_client.pull_pact(
        consumer=consumer,
        provider=provider,
        pact_version=pact_version
    )

    click.echo(result[1])


@click.command()
@click.option('--broker_url', help='Pact Broker host.')
@click.option('--consumer', help='Consumer service name.')
@click.option('--provider', help='Provider service name.')
@click.option('--user', help='Pact Broker user.')
@click.option('--password', help='Pact Broker password.')
@click.option('--pact_file', help='Path to the Pact to push.')
@click.option('--consumer_version', help='Consumer application version.')
@click.option('--pact_dir', default='.', help='Directory to save Pacts to.')
@click.option(
    '--tag', required=True, help='Consumer version tag. eg "prod".'
)
@click.option(
    '--auth',
    default=False,
    is_flag=True,
    help='Pact Broker is authenticated.'
)
def push_pact(
    broker_url,
    consumer,
    provider,
    user,
    password,
    auth,
    pact_file,
    consumer_version,
    pact_dir,
    tag
):

    broker_client = BrokerClient(
        broker_url=broker_url or settings.PACT_BROKER_URL,
        pact_dir=pact_dir or settings.PACT_DIR,
        user=user or settings.PACT_BROKER_USER,
        password=password or settings.PACT_BROKER_PASSWORD,
        authentication=auth
    )

    push_pact_result = broker_client.push_pact(
        consumer=consumer,
        provider=provider,
        pact_file=pact_file,
        consumer_version=consumer_version
    )
    click.echo(push_pact_result[1])

    tag_consumer_result = broker_client.tag_consumer(
        consumer=consumer,
        provider=provider,
        consumer_version=consumer_version,
        tag=tag
    )
    click.echo(tag_consumer_result[1])



import requests
import json

from . import urls
from requests.auth import HTTPBasicAuth


class BrokerClient:

    def __init__(
        self,
        *,
        broker_url,
        pact_dir='.',
        user=None,
        password=None,
        authentication=False
    ):
        self.broker_url = broker_url.rstrip('/')
        self.username = user
        self.password = password
        self.pact_dir = pact_dir
        self.authentication = authentication
        self._auth = None

        if self.authentication:
            if not (self.username and self.password):
                raise ValueError(
                    'When authentication is True, username and password '
                    'must be provided.'
                )
            else:
                self._auth = HTTPBasicAuth(self.username, self.password)

    def pull_pact(
        self,
        *,
        provider,
        consumer,
        tag=''
    ):
        request_url = urls.PULL_PACT_URL.format(
            broker_url=self.broker_url,
            provider=provider,
            consumer=consumer,
            tag=tag
        )

        response = requests.get(
            request_url,
            auth=self._auth
        )
        response.raise_for_status()
        saved_pact_path = self._save_pact(
            pact_json=response.json(),
            consumer=consumer,
            provider=provider
        )

        return response, f'Pact saved to {saved_pact_path}'

    def push_pact(self, *, pact_file, provider, consumer, consumer_version):
        request_url = urls.PUSH_PACT_URL.format(
            broker_url=self.broker_url,
            provider=provider,
            consumer=consumer,
            consumer_version=consumer_version
        )

        with open(pact_file) as data_file:
            pact_json = json.load(data_file)

        response = requests.put(
            request_url,
            auth=self._auth,
            json=pact_json
        )
        response.raise_for_status()
        return response, (
            f'Pact between {consumer} and {provider} pushed.'
        )

    def tag_consumer(self, *, provider, consumer, consumer_version, tag):
        request_url = urls.TAG_CONSUMER_URL.format(
            broker_url=self.broker_url,
            consumer=consumer,
            consumer_version=consumer_version,
            tag=tag
        )
        response = requests.put(
            request_url,
            headers={'Content-Type': 'application/json'},
            auth=self._auth
        )
        response.raise_for_status()
        return response, (
            f'{consumer} version {consumer_version} tagged as {tag}'
        )

    def _save_pact(self, *, pact_json, consumer, provider):
        consumer = consumer.replace(' ', '_')
        provider = provider.replace(' ', '_')
        file_path = f'{self.pact_dir}/{provider}_{consumer}.json'.lower()

        with open(file_path, 'w') as pact_file:
            json.dump(pact_json, pact_file)

        return file_path




PUSH_PACT_URL = (
    '{broker_url}/pacts/provider/{provider}/consumer/'
    '{consumer}/version/{consumer_version}'
)
PULL_PACT_URL = (
    '{broker_url}/pacts/provider/{provider}/consumer/{consumer}/latest/{tag}'
)

TAG_CONSUMER_URL = (
    '{broker_url}/pacticipants/{consumer}/versions'
    '/{consumer_version}/tags/{tag}'
)



from setuptools import setup, find_packages

with open('requirements.txt') as f:
    install_requires = f.read().splitlines()

setup(
    name='pact-broker-client',
    version='2.0.0',
    packages=find_packages(),
    include_package_data=True,
    install_requires=install_requires,
    classifiers=[
        'Programming Language :: Python :: 3.6',
    ],
    entry_points={
        'console_scripts': [
            'pull_pact=pact_broker.cli:pull_pact',
            'push_pact=pact_broker.cli:push_pact'
        ]
    },
    license='Apache 2.0',
    author='Babylon Health',
    author_email='chatbot-developers@babylonhealth.com'
)
